// --
// Core.SlickGrid.js - provides the functions for table visualisation
// Copyright (C) 2013 Меркушов Виктор merkushov.victor@gmail.cokm
// --
// $Id: Core.SlickgRID.js,v 1.0 2013/06/24 14:50:07 mb Exp $
// --
// This software comes with ABSOLUTELY NO WARRANTY. For details, see
// the enclosed file COPYING for license information (AGPL). If you
// did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
// --

"use strict";

var Core = Core || {};

/**
 * @namespace
 * @exports TargetNS as Core.SlickGrid
 * @description
 *      Contains the code for the SlickGrid functions.
 */
Core.SlickGrid = (function (TargetNS) {

    var options = {
        enableCellNavigation: true,
        showHeaderRow: true,
        headerRowHeight: 30,
        explicitInitialization: true,

        // asyncEditorLoading: false,
        forceFitColumns: true,  // растягивать столбцы по всей ширине
        enableColumnReorder: false,
        defaultColumnWidth: 120,
        fullWidthRows: true,    // стобцы занимают всё содержимое контейнера
        // multiColumnSort: true,  // сортировка
        autoHeight: true,
    };

    TargetNS.Init = function ( grid_name, columns, data, sort_info, page_info, _config ) {

        var dataView;
        var grid;
        var pager;
        var ticket_link = _config['ticket_link'];

        var columnFilters = {};

        for (var i = 0, l = columns.length; i < l; i++) {
            if ( columns[i].formatter ) {
                columns[i].formatter = eval(columns[i].formatter);
            }

            // if ( columns[i].name !== "Unread" ) {
            //     columnFilters[ columns[i].name ] = columns[i];
            // }
        }

        if ( $.cookie('sg-columns-'+grid_name) ) {
            var old_columns = Core.JSON.Parse( $.cookie('sg-columns-'+grid_name) );

            if ( columns.length == old_columns.length ) {
                var fl = 1;
                for ( var i = 0, l = columns.length; i < l; i++ ) {
                    if ( columns[i].name != old_columns[i].name ) {
                        fl = 0;
                        break;
                    }
                }

                if ( fl ) {
                    for (var i = 0, l = columns.length; i < l; i++) {
                        if ( columns[i].name == old_columns[i].name && columns[i].width !== old_columns[i].width ) {
                            columns[i].width = old_columns[i].width;
                        }
                    }
                } else {
                    $.removeCookie('sg-columns-'+grid_name);
                }
            } else {
                $.removeCookie('sg-columns-'+grid_name);
            }
        }

        if ( $.cookie('sg-page-'+grid_name) ) {
            var old_page = Core.JSON.Parse( $.cookie('sg-page-'+grid_name) );
            page_info['pageSize'] = old_page['pageSize'];
        }

        if ( $.cookie('sg-sort-'+grid_name) ) {
            var old_sort = Core.JSON.Parse( $.cookie('sg-sort-'+grid_name) );
            sort_info['sortCol'] = old_sort[0].columnId;
            sort_info['sortDir'] = old_sort[0].sortAsc;
        }

        function filter(item) {
            for (var columnId in columnFilters) {
                if (columnId !== undefined && columnFilters[columnId] !== "") {
                    var c = grid.getColumns()[grid.getColumnIndex(columnId)];
                    var field_value = item[c.field];

                    if ( columnId == "Age" ) {
                        var re_days = new RegExp( '(\\d+)(?:\\s+' + _config['day'] + ')', "gi" );
                        var days = re_days.exec(columnFilters[columnId]);

                        var re_hours = new RegExp( '(\\d+)\\s+' + _config['hour'], "gi" );
                        var hours = re_hours.exec(columnFilters[columnId]);

                        if ( days || hours ) {
                            var seconds = 0;
                            if ( days && days[1] ) {
                                seconds = seconds + days[1] * 24*60*60;
                            }
                            if ( hours && hours[1] ) {
                                seconds = seconds + hours[1] * 60*60;
                            }

                            var seconds_reserve;
                            if ( hours ) {
                                seconds_reserve = seconds + 60*60 - 1;
                            } else {
                                seconds_reserve = seconds + 24*60*60 - 1;
                            }

                            var re_sign = new RegExp( '^(>|<)', "gi" );
                            var sign = re_sign.exec(columnFilters[columnId]);

                            if ( sign ) {
                                if ( sign[1] == '>' ) {
                                    if ( field_value < seconds ) {
                                        return false;
                                    }
                                } else if ( sign[1] == '<' ) {
                                    if ( field_value > seconds ) {
                                        return false;
                                    }
                                }
                            } else if ( field_value <= seconds || field_value >= seconds_reserve ) {
                                return false;
                            }

                        } else { return true; }

                        return true;                        
                    }

                    if ( columnId == "Created" || columnId == "Changed" ) {
                        var re = /(\d+)-(\d+)-(\d+)\s(\d+:\d+:\d+)/;
                        field_value = field_value.replace( re, "$3-$2-$1 $4" );
                    }

                    var re = new RegExp( columnFilters[columnId], "i" );
                    if ( !field_value.match(re) ) {
                        return false;
                    }
                    
                }
            }
            return true;
        }

        function comparer(a, b) {
            var x = a[sort_info['sortCol']], y = b[sort_info['sortCol']];
            return (x == y ? 0 : (x > y ? 1 : -1));
        };

        function FormatterAge(row, cell, value, columnDef, dataContext) {
            var mod = value % (24*60*60);
            var days = ( value - mod ) / (24*60*60);
            var hours = ( mod - (mod % (60*60)) ) / (60*60);

            var new_value = '';
            if ( days ) {
                new_value = days + ' ' + _config['day'];
            }
            if ( hours ) {
                new_value = new_value + ' ' + hours + ' ' + _config['hour'];
            }

            return  new_value;
        }

        function FormatterDate(row, cell, value, columnDef, dataContext) {
            var re = /(\d+)-(\d+)-(\d+)\s(\d+:\d+:\d+)/;
            var new_value = value.replace( re, "$3-$2-$1 $4" );

            return new_value;
        }

        function FormatterUnread(row, cell, value, columnDef, dataContext) {
            var str;
            if ( value ) {
                str = '<div title="Непрочитанное сообщение" class="UnreadArticles Small"><span class="UnreadArticles Important">Непрочитанное сообщение</span></div>';
            }

            return str;
        }

        dataView    = new Slick.Data.DataView();
        grid        = new Slick.Grid('#gridContainer-'+grid_name, dataView, columns, options);
        pager       = new Slick.Controls.Pager(dataView, grid, $('#pager-'+grid_name));

        grid.registerPlugin( new Slick.AutoTooltips({ enableForHeaderCells: true }) );
        grid.setSortColumn(sort_info['sortCol'], sort_info['sortDir']);

        grid.onClick.subscribe(function (e, args) {
            window.location = ticket_link + data[args.row].id;
            return false;
        });

        grid.onSort.subscribe(function (e, args) {
            // console.log(args); 

            var s = grid.getSortColumns();

            sort_info['sortDir'] = s[0].sortAsc ? 1 : -1;
            sort_info['sortCol'] = s[0].columnId;

            if ($.browser.msie && $.browser.version <= 8) {
                var percentCompleteValueFn = function () {
                    var val = this["percentComplete"];
                    if (val < 10) {
                        return "00" + val;
                    } else if (val < 100) {
                        return "0" + val;
                    } else {
                        return val;
                    }
                };

                dataView.fastSort(
                    (
                        sort_info['sortCol'] == "percentComplete")
                            ? percentCompleteValueFn 
                            : sort_info['sortCol'], 
                        s[0].sortAsc
                    );
            } else {
                dataView.sort(comparer, s[0].sortAsc);
            }

            $.cookie('sg-sort-'+grid_name, Core.JSON.Stringify( grid.getSortColumns() ), { expires: 365, path: '/' });
        });

        grid.onColumnsResized.subscribe(function (e, args) {
            var list = grid.getColumns();
            var new_list = [];
            for (var i = 0, l = list.length; i < l; i++) {
                new_list[i] = { name: list[i].name, width: list[i].width };
            }
            $.cookie('sg-columns-'+grid_name, Core.JSON.Stringify( new_list ), { expires: 365, path: '/' });
        });
        dataView.onPagingInfoChanged.subscribe(function (e, args) {
            $.cookie('sg-page-'+grid_name, Core.JSON.Stringify( dataView.getPagingInfo() ), { expires: 365, path: '/' });
        });

        // wire up model events to drive the grid
        dataView.onRowCountChanged.subscribe(function (e, args) {
            grid.updateRowCount();
            grid.render();
        });

        dataView.onRowsChanged.subscribe(function (e, args) {
            grid.invalidateRows(args.rows);
            grid.render();
        });

        $(grid.getHeaderRow()).delegate(":input", "change keyup", function (e) {
            var columnId = $(this).data("columnId");
            if (columnId != null) {
                columnFilters[columnId] = $.trim($(this).val());
                dataView.refresh();
            }
        });

        grid.onHeaderRowCellRendered.subscribe(function(e, args) {
            if ( args.column.field == "Unread" ) {
                return;
            }

            $(args.node).empty();
            $("<input type='text'>")
               .data("columnId", args.column.id)
               .val(columnFilters[args.column.id])
               .appendTo(args.node);
        });

        grid.init();

        // initialize the model after all the events have been hooked up
        dataView.beginUpdate();
        dataView.setItems(data);
        dataView.setPagingOptions(page_info);
        dataView.sort(comparer, sort_info['sortDir']);
        dataView.setFilter(filter);
        dataView.endUpdate();

    };

    return TargetNS;
}(Core.SlickGrid || {}));

// (function () {
// }());
